package nl.uvt.locator;

import java.net.*;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import nu.xom.Document;
import nu.xom.Nodes;

public class LinkDbConnection {
	List<LinkInfo> query( QueryData pQ ) throws LocatorException{
		List<LinkInfo> linkInfo = null;
		if ( pQ.issn.length() > 0 ){
			linkInfo = queryLinkDbIssn( pQ );   
		}
		else if ( pQ.title.length() > 0 ){
			linkInfo = queryLinkDbTitle( pQ );   
		}
		return linkInfo;
	}
	
	private List<LinkInfo> queryLinkDbIssn( QueryData pQ ) throws LocatorException {
		List<LinkInfo> linkResult = null;
		nu.xom.Document result;
		String linkDbUrl = Locator.locatorConfig.getProperty("linkDbUri");
		String query = "";
		try {
			// try linkDb with issn
			Properties urlProps = new Properties( Locator.sruDefaultProps );
			query = URLEncoder.encode( pQ.issn, "ISO-8859-1" );
			urlProps.setProperty("query", "bath.issn="+query );
			linkDbUrl = Locator.createUrlString( linkDbUrl, urlProps );
			Locator.logger.debug("Try URL: " + linkDbUrl );
			result = Locator.httpGetDocument( linkDbUrl, Locator.httpConfig );
//			Locator.logger.debug( "ISSN search returns:");
//			Locator.logger.debug( result.toXML() );
			int hits = Locator.extractHits( result );
			Locator.logger.debug("linkdb numOfRecords=" + hits );
			if ( hits > 0 ){
				linkResult = getLinkDbResults( result );
			}
		}
		catch (Exception e)  {
			throw new LocatorException( "URL: '" + linkDbUrl + "' failed: " + e.getMessage());
		}
		return linkResult;
	}

	private  List<LinkInfo> queryLinkDbTitle( QueryData pQ ) throws LocatorException {
		List<LinkInfo> linkResult = null;
		nu.xom.Document result;
		String linkDbUrl = Locator.locatorConfig.getProperty("linkDbUri");
		try {
			// try linkDb with Title
			Properties prop = new Properties( Locator.sruDefaultProps );
			String titleParts[] = pQ.title.split( ":", 2 );
			String mainTitle = pQ.title;
			if ( titleParts.length > 1 ){
				for( int i=0; i < titleParts.length; ++i )
					Locator.logger.debug( "part["+i+"]= '"+titleParts[i] +"'");
				mainTitle = titleParts[0];
				// HACK
				// Although a title like "Work, employment & society" is stored in Oracle 
				// as  <title>Work, employment &amp; society</title>
				// you MUST use a query like "Work, employment & society"
				// using ".. &amp; ..' will find no match
				// example:  
				// it='Industriel Districts and Inter-Firm Co-operation in Italy.' in olc
				// (including typo!)
				// maybe we need to ampdecode more stuff?
				// mainTitle = mainTitle.replace("&amp;", "&");
			}
			prop.setProperty("query", "dc.title=\"" + mainTitle + "\"" );
			linkDbUrl = Locator.createUrlString(linkDbUrl , prop);
			Locator.logger.debug("Try URL: " + linkDbUrl );
			result = Locator.httpGetDocument( linkDbUrl, Locator.httpConfig);
//			logger.debug( "linkDb Title search returns:");
//			logger.debug( result.toXML() );
			int hits = Locator.extractHits( result );
			Locator.logger.debug("linkdb numOfRecords=" + hits );
			if ( hits > 0 ){
				linkResult = getLinkDbResults( result );
			}
		}
		catch (Exception e) 
		{
			throw new LocatorException( "URL:'" + linkDbUrl + "' failed: "+ e.getMessage());
		}
		return linkResult;
	}

	private List<LinkInfo> getLinkDbResults( Document doc ){
		List<LinkInfo> linkResult = null;
		Nodes records = doc.query("//srw:recordData/record|//srw:recordData/rec", Locator.sruContext);
		if ( records.size() > 0 ){
			linkResult = new LinkedList<LinkInfo>();
			for ( int i=0; i < records.size(); ++i ){
				LinkDbInfo newInfo = new LinkDbInfo();
				newInfo.fill( records.get(i) );
				Locator.logger.debug( newInfo.toString());
				linkResult.add( newInfo );
			}
		}
		else{
			Locator.logger.warn("no record fields found!");
			Locator.logger.warn(doc.toXML());
		}
		return linkResult;
	}


}
